#include <ctype.h>
#include "Global.h"
#include "Util.h"
#include "Object.h"
#include "Hero.h"

shAttack *ExplodingEnergyCellDamage =
    newBurstAttack (shAttack::kBlast, 0, 1, kElectrical, 3, 6);


shAttack *RestrainingBoltAttack =
    newSingleAttack (shAttack::kAttach, 0, kNoEnergy, 0, 1);

int
shCreature::attemptRestraining (shObject *bolt)
{
    if (isA (kMonClerkbot) || isA (kMonDocbot)) { /* nice try */
        I->p ("The restraining bolt is vaporized by %s's "
              "anti-shoplifting circuits!", the ());
        newEnemy (&Hero);
        delete bolt;
        return 0;
    }
    addObjectToInventory (bolt);
    if (!isPet ()) ((shMonster *) this) -> makePet ();
    return 1;
}

static int
useRestrainingBolt (shObject *bolt)
{
    shDirection dir;
    int x = Hero.mX;
    int y = Hero.mY;
    shMonster *c;
    int difficulty;

    dir = I->getDirection ();
    switch (dir) {
    case kNoDirection:
    case kUp:
    case kDown:
        return 0;
    default:
        if (!Level->moveForward (dir, &x, &y)) {
            return 0;
        }
        c = (shMonster *) Level->getCreature (x, y);
        if (!c || !c->isRobot ()) {
            I->p ("Restraining bolts only work on bots and droids.");
            return 0;
        }
        if (c->isPet ()) {
            I->p ("You've already affixed a restraining bolt to %s.",
                  THE (c));
            return 0;
        }

        /* low-level bots should be automatic, warbots should be quite
           difficult to tame! */
        difficulty = bolt->isOptimized () ? 6 :
                     bolt->isBuggy ()     ? 2 :
                                            3;

        if (c->mCLevel > difficulty && RNG (c->mCLevel) > difficulty) {
            I->p ("You miss %s.", THE (c));
            c->newEnemy (&Hero);
        } else {
            Hero.usedUpItem (bolt, 1, "attach");
            bolt = Hero.removeOneObjectFromInventory (bolt);
            I->p ("You attach the restraining bolt to %s.", THE (c));
            c->attemptRestraining (bolt);
        }
        return FULLTURN;
    }
}


static void
flail (shObject *md, const char *what)
{
    I->p ("You flail %s with your limp %s.", what, md->getShortDescription ());
    I->p ("For some reason it doesn't appear to improve its condition.");
}


static int
repairObject (shObject *tool)
{
    shObjectVector v;
    selectObjectsByFunction (&v, Hero.mInventory, &shObject::isDamaged);
    if (!v.count ()) {
        I->p ("You have nothing to repair.");
        return 0;
    }
    shObject *obj = Hero.quickPickItem (&v, "repair", 0);
    shObject *material;
    if (!obj) return 0;
    if (obj->myIlk ()->mMaterial == kFleshy) {
        I->p ("Repair a piece of meat?  How?");
        return 0;
    }
    if (!obj->isIlkKnown () ||
        (!obj->isAppearanceKnown () && /* Cannot see it and ... */
         /* ... seeing would make a difference. */
         strcmp (obj->myIlk ()->mAppearance, obj->myIlk ()->mVagueName))) {
        I->p ("But you don't surely know what it really is.");
        return 0;
    }
    if (obj->isWorn ()) {
        I->p ("Take it off first.");
        return 0;
    }
    if (tool->isA (kObjMechaDendrites) && !tool->isWorn ()) {
        flail (tool, obj->theQuick ());
        return FULLTURN;
    }
    material = Hero.quickPickItem (Hero.mInventory, "get material from", 0);
    if (!material) {
        I->p ("You cannot repair without spare parts.");
        return 0;
    }
    if (obj == material) {
        I->p ("You cannot repair %s with itself.", THE (obj));
        return 0;
    } else if (material->isWorn ()) {
        I->p ("Take it off first.");
        return 0;
    } else if (!material->isAppearanceKnown () &&
         strcmp (obj->myIlk ()->mAppearance, obj->myIlk ()->mVagueName))
    {
        I->p ("But you don't surely know what the material is made of.");
        return 0;
    } else if (material->myIlk ()->mMaterial == kFleshy) {
        I->p ("Piece of meat as material? What are you thinking?");
        return 0;
    } /* Now the complex rules begin. */
    if (obj->isThrownWeapon ()) {
        if (!material->isThrownWeapon ()) {
            I->p ("%s is too dissimilar.", THE (material));
            return 0;
        }
    } else if (obj->isMeleeWeapon ()) {
        if (obj->myIlk ()->mMaterial != material->myIlk ()->mMaterial) {
            I->p ("To repair a melee weapon you need more of the material its made of.");
            return 0;
        }
    } else if (obj->isAimedWeapon ()) {
        if (!(material->isAimedWeapon ())) {
            I->p ("You need to repair it with another ranged weapon.");
            return 0;
        }
        if ((obj->isA (kObjLightFlamethrower) ||
             obj->isA (kObjFlamethrower)) &&
            (!material->isA (kObjLightFlamethrower) &&
             !material->isA (kObjFlamethrower)))
        {
            I->p ("To repair a flamethrower you need another one.");
            return 0;
        } else {
            if (obj->myIlk ()->mParent != material->myIlk ()->mParent) {
                I->p ("These weapons belong to different class.");
                return 0;
            }
        }
    } else if (obj->isA (kArmor)) {
        if (obj->isA (kObjGenericJumpsuit)) {
            if (!material->isA (kObjGenericJumpsuit)) {
                I->p ("You need another jumpsuit.");
                return 0;
            }
            if (obj->isA (kObjChameleonJumpsuit) &&
                !material->isA (kObjChameleonJumpsuit))
            {
                I->p ("Chameleon suits must be repaired using"
                      " patches from another one.");
                I->p ("Otherwise its camouflaging properties"
                      " would be diminished.");
                return 0;
            } else if (obj->isA (kObjRadiationJumpsuit) &&
                       !material->isA (kObjRadiationJumpsuit))
            {
                I->p ("Radiation suits must be repaired using"
                      " patches from another one.");
                I->p ("Otherwise its protective properties"
                      " would be diminished.");
                return 0;
            } /* Otherwise ok. */
        } else if (obj->isA (kObjGenericArmor) ||
                   obj->isA (kObjGenericHelmet))
        {   /* Compatibility rules:
                - type match: helmet <-> helmet or body armor <-> body armor
                - powered match: both powered or none
                - material type match */
            if (obj->myIlk ()->mParent != material->myIlk ()->mParent) {
                I->p ("Armor types do not match.");
                return 0;
            }
            int powered = obj->isPoweredArmor () + material->isPoweredArmor ();
            if (powered % 2) {
                I->p ("One of those armor pieces is powered.");
                I->p ("Either both of those should be powered or none of them.");
                return 0;
            }
            if (obj->myIlk ()->mMaterial != material->myIlk ()->mMaterial) {
                I->p ("The material types do not match.");
                return 0;
            }
        } else {
            I->p ("This armor cannot be repaired.");
            return 0;
        }
    } else {
        I->p ("This object does not have repair handle. Plase report this bug.");
        return 0;
    }
    int rad = obj->isRadioactive () + material->isRadioactive ();
    /* Finally, repair. */
    Hero.removeObjectFromInventory (material);
    delete material;
    if (rad) {
        shObjectIlk *rad_gr = AllIlks[kObjRadGrenade];
        for (int i = 0; i < rad; ++i) {
            Hero.sufferDamage (rad_gr->mMissileAttack);
        }
    }
    if (sportingD20 () + Hero.getSkillModifier (kRepair) > 15) {
        obj->mDamage = 0;
        I->p ("Now %s looks as good as new!", THE (obj));
    } else {
        I->p ("Your repair attempt is a failure.");
    }
    return 1;
}


int
makeRepair (shObject *tool)
{
    if (Hero.mWeapon && Hero.mWeapon->isWeldedWeapon ()) {
        I->p ("You cannot free both your hands.");
        return 0;
    }
    shDirection dir;
    int x = Hero.mX, y = Hero.mY;
    shFeature *f;
    shMonster *c;
    shObjectVector *v;
    int ducttape = 0;
    int flailing = 0;
    int score = sportingD20 () + Hero.getSkillModifier (kRepair)
                + 2 * tool->mBugginess;

    if (tool->isA (kObjDuctTape)) {
    /* At first I had this as a penalty, but duct-tape holds the universe
       together, right?  Plus this might make for better game balance.  I think
       it's no fun if the one-time use items suck compared to reusable ones...
       -CADV
    */
        score += 3;
        ducttape = 1;
    } else if (tool->isA (kObjMonkeyWrench)) {
        score += tool->mEnhancement;
    }
    if (tool->isA (kObjMechaDendrites) && !tool->isWorn ()) {
        flailing = 1;
    }

    I->p ("Select yourself to repair items in inventory.");
    dir = I->getDirection ();
    switch (dir) {
    case kNoDirection:
        return 0;
    case kUp:
        if (Level->mSquares[Hero.mX][Hero.mY].mTerr == kBrokenLightAbove) {
            if (flailing) {
                flail (tool, "the broken light");
                return FULLTURN;
            } else if (Hero.mProfession == Janitor) {
                int room = Level->getRoomID (Hero.mX, Hero.mY);
                int sx = Hero.mX, sy = Hero.mY, ex = Hero.mX, ey = Hero.mY;
                Level->getRoomDimensions (room, &sx, &sy, &ex, &ey);
                for (int y = sy; y <= ey; ++y)
                    for (int x = sx; x <= ex; ++x)
                        Level->setLit (x, y);
                Level->mSquares[Hero.mX][Hero.mY].mTerr = kFloor;
                I->p ("You replace the missing lightbulb.");
                if (tool->isA (kObjMechaDendrites)) {
                    I->p ("Screwing it with %s was fun.", YOUR (tool));
                }
                Hero.earnXP (Level->mDLevel);
                break;
            } else {
                I->p ("The lightbulb is missing.  Call janitor to bring a spare.");
                return 0;
            }
        }
        I->p ("There's nothing to repair on the ceiling.");
        return 0;
    case kOrigin:
        if (!repairObject (tool)) {
            return 0;
        } else {
            break;
        }
    default:
        if (!Level->moveForward (dir, &x, &y)) {
            I->p ("There's nothing there to repair.");
            return 0;
        }
        c = (shMonster *) Level->getCreature (x, y);
        if (c && dir != kDown) {
            /* repair a pet droid */
            const char *who = THE (c);
            if (c->isHero ()) {
                I->p ("You can't make repairs to yourself!");
                return 0;
            } else if (!c->isRobot ()) {
                I->p ("You can't make repairs to %s!", who);
                return 0;
            } else if (c->mHP == c->mMaxHP) {
                I->p ("%s doesn't appear to be damaged.", who);
                return 0;
            } else if (!c->isPet ()) { /* Replace with mDisposition check? */
                I->p ("%s won't cooperate with your attempt to repair it.",
                      who);
                return FULLTURN;
            } else if (flailing) {
                flail (tool, who);
                return FULLTURN;
            } else if (score < 15) {
                I->p ("Your repair attempt is a failure.");
            } else {
                c->mHP += score - 15 + RNG (1, 3);
                if (c->mHP >= c->mMaxHP) {
                    c->mHP = c->mMaxHP;
                    I->p ("Now %s looks as good as new!", who);
                } else {
                    I->p ("You repair some of the damage on %s.", who);
                }
            }
            break;
        }
        if ((v = Level->getObjects (x, y))) {
            int i;
            shObject *obj;
            int nx = x;
            int ny = y;
            int repaired = 0;

            for (i = 0; i < v->count (); i++) {
                obj = v->get (i);
                if (obj->isA (kObjWreck)) {
                    if (nx == Hero.mX && ny == Hero.mY &&
                        -1 == Level->findNearbyUnoccupiedSquare (&nx, &ny))
                    {
                        I->p ("There is %s here, but you need more room "
                              "to repair it.", AN (obj));
                        break;
                    }
                    if (I->yn ("Repair %s?", THE (obj))) {
                        if (flailing) {
                            flail (tool, THE (obj));
                            return FULLTURN;
                        } else if (score < 20) {
                            I->p ("Your repair attempt is a failure.");
                        } else {
                            shMonster *bot = new shMonster (obj->mWreckIlk);
                            /* Prevent XP farming. */
                            bot->mConditions |= kNoExp;
                            /* Prevent drop farming. */
                            for (int i = bot->mInventory->count () -1; i >= 0; --i) {
                                shObject *obj = bot->mInventory->get (i);
                                bot->removeObjectFromInventory (obj);
                                delete obj;
                            }
                            Level->putCreature (bot, nx, ny);
                            I->p ("You bring %s back on-line!", THE (obj));
                            v->remove (obj);
                            delete obj;
                        }
                        repaired = 1;
                        break;
                    }
                }
            }
            if (repaired) break;
        }

        if ((f = Level->getFeature (x, y))) {
            if ((shFeature::kDoorOpen == f->mType ||
                shFeature::kDoorClosed == f->mType) &&
                shFeature::kBerserk & f->mDoor.mFlags &&
                !f->mTrapUnknown)
            {
                if (flailing) {
                    flail (tool, "the door");
                } else if (score >= 12) {
                    f->mDoor.mFlags &= ~shFeature::kBerserk;
                    I->p ("You repair the malfunctioning door.");
                    if (Hero.mProfession == Janitor) {
                        /* A shout out to ADOM. :) */
                        I->p ("You feel like behaving properly.");
                        /* For doing what you should be. */
                        Hero.earnXP (Level->mDLevel);
                    }
                    Level->remember (x, y, f); /* For blind chars. */
                } else {
                    I->p ("You repair attempt is a failure.");
                }
                break;
            } else if (f->isLockBrokenDoor ()) {
                if (flailing) {
                    flail (tool, "the broken lock");
                } else if (score >= 15) {
                    f->mDoor.mFlags &= ~shFeature::kLockBroken;
                    I->p ("You repair the broken lock.");
                    // No Janitor bonus. This could be abused.
                } else {
                    I->p ("You repair attempt is a failure.");
                }
                break;
            } else if ((shFeature::kPit == f->mType ||
                        shFeature::kAcidPit == f->mType ||
                        shFeature::kHole == f->mType) && ducttape)
            {
                if (Hero.mZ == -1) {
                    I->p ("Climb out first.");
                    return 0;
                }
                I->p ("You tape %s opening. Looks almost like a solid ground.",
                    f->mType == shFeature::kHole ? "hole" : "pit");
                /* Make sure no intelligent enemy saw you at work. */
                for (int i = 0; i < Level->mCrList.count (); ++i) {
                    shCreature *c = Level->mCrList.get (i);
                    if (c && c->mAbil.mInt > 6 && c->canSee (&Hero) &&
                        !c->isHero () && !c->isPet ())
                    {
                        if (Hero.canSee (c)) {
                            I->p ("%s watches you closely.", THE (c));
                        }
                        break;
                    }
                } /* It shall be conveniently assumed monsters had plenty
                     of time to forget the trap even existed. */
                f->mTrapMonUnknown = 1;
                break;
            } else {
                I->p ("It ain't broke!");
                return 0;
            }
        }
        I->p ("There's nothing there to repair.");
        return 0;
    }
    if (tool->isA (kObjMonkeyWrench) ||
        (tool->isA (kObjMechaDendrites) && !flailing)) {
        /* This is intentionally a ripoff in case of wrench. */
        Hero.employedItem (tool, 10 BZ);
    }
    if (flailing) {
        Hero.employedItem (tool, 0 BZ); /* This leads to a YAFM. */
    }
    if (ducttape) {
        /* superglue will get used up, too, but that's taken care of by
           the useCanister() function
         */
        Hero.useUpOneObjectFromInventory (tool);
    }
    /* yeah, realistically this should take longer, but that wouldn't
       be very fun for the player I think. */
    return FULLTURN;
}


int
shHero::useKey (shObject *key, shFeature *door)
{
    int elapsed = 0;
    int locked = door->isLockedDoor ();
    const char *does_what = locked ? "unlocks" : "locks";
    shObjectIlk *cardneeded = door->keyNeededForDoor ();

    if (shFeature::kDoorHiddenHoriz == door->mType ||
        shFeature::kDoorHiddenVert == door->mType)
    {
        I->p ("There's no lock there.");
        return 0;
    }
    if (door->isLockBrokenDoor ()) {
        I->p ("The lock on this door is broken.");
        return QUICKTURN;
    }
    if (!cardneeded && !door->isRetinaDoor ()) {
        I->p ("There is no lock on this door.");
        return QUICKTURN;
    }
    if (door->isOpenDoor ()) {
        I->p ("You have to close it first.");
        return 0;
    }

    if (cardneeded && key->isA (kObjGenericKeycard)) {
        const char *key_d = key->getShortDescription ();
        if (isBlind ()) {
            if (!key->isBuggy () &&
                (key->isA (kObjMasterKeycard) ||
                 key->myIlk () == cardneeded ||
                 key->isCracked ()))
            {
                I->p ("You swipe your %s and the door lock accepts it.", key_d);
                if (key->isAppearanceKnown () && /* You know keycard color. */
                /* The keycard is not cracked or you are unaware of the fact. */
                    (!key->isCracked () || !key->isCrackedKnown ()) &&
                /* This is not master keycard or you are unaware of the fact. */
                    (!key->isA (kObjMasterKeycard) || !key->isIlkKnown ()))
                { /* You infer the door must be color of keycard. */
                    int codeflag = door->mDoor.mFlags & shFeature::kColorCode;
                    Level->mRemembered[door->mX][door->mY].mDoor &= ~shFeature::kColorCode;
                    Level->mRemembered[door->mX][door->mY].mDoor |= codeflag;
                }
            } else {
                I->p ("You swipe your %s but it seems not to work.", key_d);
                return QUICKTURN;
            }
        } else if (key->isBuggy () && key->isBugginessKnown ()) {
            I->p ("But your %s is buggy.", key_d);
            return 0;
        } else if (key->myIlk () == cardneeded) {
            if (key->isBuggy ()) {
                I->p ("You swipe %s but the door does not react.", key_d);
                I->p ("It must be buggy.");
                key->setBugginessKnown ();
                return QUICKTURN;
            }
            I->p ("You swipe your %s and the door %s.", key_d, does_what);
        } else if (key->isBuggy ()) {
            I->p ("You swipe your %s but nothing happens.", key_d);
            if ((key->isA (kObjMasterKeycard) && key->isIlkKnown ()) ||
                (key->isCracked () && key->isCrackedKnown ()))
            {
                I->p ("Damn it!  This keycard must be buggy.");
                key->setBugginessKnown ();
            } else {
                I->p ("Obviously the keycard and lock colors do not match.");
            }
            return QUICKTURN;
        } else if (key->isA (kObjMasterKeycard)) {
            I->p ("You swipe your %s and the door %s.", key_d, does_what);
            if (!key->isIlkKnown () && key->myIlk () != cardneeded) {
                I->p ("This must be the master keycard.");
                key->setIlkKnown ();
            }
        } else if (key->isCracked ()) {
            if (key->isCrackedKnown ()) {
                I->p ("You swipe your %s and the door %s.", key_d, does_what);
            } else {
                I->p ("You swipe your %s.  To your surprise the door %s.",
                    key_d, does_what);
                I->p ("This keycard must be cracked.");
                key->setCrackedKnown ();
            }
        } else {
            I->p ("You swipe your %s but nothing happens.", key_d);
            I->p ("Obviously the keycard and lock colors do not match.");
            if (key->isBugginessKnown ()) key->setCrackedKnown ();
            return QUICKTURN;
        }
        elapsed = QUICKTURN;
    } else if (key->isA (kObjLockPick)) {
        int score = sportingD20 () + Hero.getSkillModifier (kOpenLock);
        if (door->isRetinaDoor ())
            score -= 10;
        if (!locked)
            score += 4;
        if (score >= 20) {
            if (locked) {
                I->p ("You run a bypass on the locking mechanism.");
                if (door->isAlarmedDoor () && mProfession == Ninja) {
                    I->p ("You also disable the door's alarm system.  Ninja'd!");
                    door->mDoor.mFlags &= ~shFeature::kAlarmed;
                    Hero.earnXP (Level->mDLevel);
                }
            } else {
                I->p ("You lock the door.");
            }
            elapsed = FULLTURN;
        } else {
            if (door->isRetinaDoor ())
                I->p ("It's very difficult to bypass "
                      "the retina scanner.");
            if (door->isAlarmedDoor () && score <= 10) {
                I->p ("You set off an alarm!");
                Level->doorAlarm (door);
            } else {
                I->p ("You fail to defeat the lock.");
            }
            return FULLTURN;
        }
    } else if (door->isRetinaDoor ()) {
        I->p ("You need the proper retina to unlock this door.");
    } else {
        if (isBlind ()) {
            I->p ("Nothing happens.");
        } else {
            char *buf = GetBuf ();
            strcpy (buf, door->getDoorLockName ());
            /* Only orange key needs to be preceded by 'an'. */
            const char *an = buf[0] == 'o' ? "an" : "a";
            I->p ("You need %s %s keycard to %s this door.", an, buf,
                  locked ? "unlock" : "lock");
        }
        return QUICKTURN;
    }
    if (locked) {
        door->unlockDoor ();
    } else {
        door->lockDoor ();
    }
    return elapsed;
}


static int
useKeyTool (shObject *key)
{
    shDirection dir;
    int x = Hero.mX;
    int y = Hero.mY;
    shFeature *f;

    dir = I->getDirection ();
    switch (dir) {
    case kNoDirection:
        return 0;
    case kUp:
        I->p ("There's no lock on the ceiling.");
        return 0;
    default:
        if (!Level->moveForward (dir, &x, &y)) {
            I->p ("There's no lock there.");
            return 0;
        }
        if ((f = Level->getFeature (x, y)) && f->isDoor ()) {
            return Hero.useKey (key, f);
        }
        I->p ("There's no lock there.");
        return 0;
    }
}


static int
useComputer (shObject *computer)
{
    if (Hero.is (kUnableCompute)) {
        I->p ("You dare to read the Nam-Shub?!  Defiler.");
        return 0;
    }

    shObjectVector v;
    selectObjects (&v, Hero.mInventory, kFloppyDisk);
    shObject *obj = Hero.quickPickItem (&v, "execute", 0);
    if (!obj) {
        return 0;
    }

    return obj->runFloppyDisk (computer);
}


static int
useMedicomp (shObject *medicomp)
{
    const int PLAGUE_COST = 10;
    const int RESTORE_COST = 4;
    if (medicomp->isOptimized ()) {
        medicomp->setChargeKnown ();
    }
    if (!medicomp->mCharges) {
        I->p ("The medicomp is out of charges.");
        int known = medicomp->isChargeKnown ();
        medicomp->setChargeKnown ();
        return known ? 0 : HALFTURN;
    }
    if (Hero.is (kPlagued)) {
        if (medicomp->mCharges >= PLAGUE_COST) {
            medicomp->mCharges -= PLAGUE_COST;
            Hero.restoration ();
            return LONGTURN;
        } else {
            I->p ("Not enough charges to cure plague.");
            int known = medicomp->isChargeKnown ();
            medicomp->setChargeKnown ();
            return known ? 0 : HALFTURN;
        }
    }
    if (Hero.mHP < Hero.mMaxHP || Hero.is (kViolated) || Hero.is (kStunned) ||
        Hero.is (kConfused) || Hero.getStoryFlag ("brain incision"))
    {
        --medicomp->mCharges;
        Hero.healing (0);
        return LONGTURN;
    }
    if (Hero.needsRestoration ()) {
        if (medicomp->mCharges >= RESTORE_COST) {
            medicomp->mCharges -= RESTORE_COST;
            Hero.restoration ();
            return LONGTURN;
        } else {
            I->p ("Not enough charges to restore abilities.");
            int known = medicomp->isChargeKnown ();
            medicomp->setChargeKnown ();
            return known ? 0 : HALFTURN;
        }
    }
    I->p ("Nothing to cure.");
    return 0;
}


static int
useTricorder (shObject *tricorder)
{
    static const char *creatureTypeName[kMaxCreatureType] =
    {
        "unliving",
        "unliving",
        "unliving",
        "unliving",
        "egg",
        "ooze",
        "cyborg",
        "aberration",
        "animal",
        "alien",
        "beast",
        "humanoid",
        "mutant humanoid",
        "insect",
        "unknown lifeform", /* outsider */
        "vermin",
        "zerg"
    };

    if (!tricorder->isIlkKnown () || !tricorder->isAppearanceKnown ()) {
        I->p ("It is a tricorder.");
        tricorder->setIlkKnown ();
        I->pause ();
    }
    shDirection dir;
    int x = Hero.mX;
    int y = Hero.mY;
    shMonster *c;

    if (tricorder->isBuggy ()) {
        I->p ("It appears to be broken.");
        tricorder->setBugginessKnown ();
        return 0;
    }
    dir = I->getDirection ();
    switch (dir) {
    case kNoDirection:
    case kUp:
        return 0;
    case kDown: {
            shFeature *f = Level->getFeature (x, y);
            if (f && f->mType == shFeature::kVat) {
                if (!Hero.loseEnergy (5)) {
                    I->p ("You have not enough energy to scan this vat.");
                    return 0;
                }
                /* Intentionally says nothing about radioactiveness.
                   Use geiger counter for this purpose. */
                f->mVat.mAnalyzed = 1;
                I->p ("According to %s this is %s.",
                    YOUR (tricorder), f->getDescription ());
                I->drawSideWin ();
                return LONGTURN;
            }
            return 0;
        }
    default:
        if (!Level->moveForward (dir, &x, &y)) {
            return 0;
        }
        if (!Hero.loseEnergy (1)) {
            I->p ("You have no energy for %s.", YOUR (tricorder));
            return 0;
        }
        c = (shMonster *) Level->getCreature (x, y);
        if (!c) {
            I->p ("You detect no creature there.");
            I->drawSideWin ();
            return 0;
        };
        const char *desc;
        if (Hero.isBlind ()) {
            desc = creatureTypeName[c->mType];
        } else {
            desc = c->getDescription ();
        }
        c->mHidden = 0;

        I->p ("%s HP:%d/%d AC: %d", desc, c->mHP, c->mMaxHP, c->getAC ());
        I->drawSideWin ();
        /* Creature diagnosed must stay still. */
        if (c->isHero () || c->isPet () || c->isSessile () ||
            c->is (kParalyzed) || c->is (kAsleep) || c->isTrapped ())
        {
            bool wasbugknown = tricorder->isBugginessKnown ();
            tricorder->setBugginessKnown ();
            if (tricorder->isDebugged ()) {
                if (!wasbugknown) {
                    I->p ("Optimize your tricorder to perform full diangostic scans.");
                }
            } else if (I->yn ("Perform full diagnostic scan?")) {
                if (!Hero.loseEnergy (9)) {
                    I->p ("You have no energy to perform full scan.");
                    return 0;
                }
                /* Shopkeepers charge only for using diagnosis. On the other
                   hand it is cheaper than docbot service so worth it. */
                Hero.employedItem (tricorder, 100 BZ);
                I->drawSideWin ();
                /* Diagnosis screen would obscure this information. */
                I->pause ();
                c->doDiagnostics (tricorder->mBugginess);
                return LONGTURN;
            }
        }
    } /* Takes no time to use to be useful in combat. */
    return 0;
}


static int
useDroidCaller (shObject *obj)
{
    shCreature *c;
    shVector <shCreature *> clist;

    if (Level->isInGarbageCompactor (Hero.mX, Hero.mY) &&
        !obj->isBuggy () && Level->mCompactorState < 10 &&
        Level->mCompactorState > 0)
    {
        for (int i = 0; i < Hero.mPets.count (); ++i) {
            shCreature *pet = Hero.mPets.get (i);
            /* R2D2! Turn off the garbage compactor at the sewer plant! */
            if (pet->isA (kMonAstromechDroid)) {
                I->p ("You call upon your astromech droid for help.");
                if (Hero.tryToTranslate (pet)) {
                    if (Level->mCompactorHacked == 0) {
                        I->p ("The R2 unit on %s %d acknowledges your request.",
                              pet->mLevel->mName, pet->mLevel->mDLevel);
                    } else {
                        I->p ("The R2 unit on %s %d is working on it.",
                              pet->mLevel->mName, pet->mLevel->mDLevel);
                    }
                } else {
                    I->p ("You receive a torrent of beeps and chirps"
                          " in response.");
                }
                Level->mCompactorHacked = 1; /* Hero is safe. */
                return HALFTURN;
            }
        }
    }
    I->p ("%s produces a strange whistling sound.", THE (obj));

    for (int i = 0; i < Level->mCrList.count (); ++i) {
        c = Level->mCrList.get (i);
        if (   (obj->isBuggy () && c->isRobot ())
            || (!obj->isBuggy () && c->isPet ()))
        {
            int x = Hero.mX;
            int y = Hero.mY;
            if (!obj->isOptimized ()) {
                x = x + RNG(7) - 3;
                y = y + RNG(5) - 2;
            }

            if (!Level->findNearbyUnoccupiedSquare (&x, &y)) {
                c->transport (x, y, 100, 1);
                if (Hero.canSee (c) && !c->isPet ()) {
                    obj->setBugginessKnown ();
                }
                if (c->isA (kMonClerkbot) ||
                    c->isA (kMonGuardbot) ||
                    c->isA (kMonSecuritron) ||
                    c->isA (kMonDocbot) ||
                    c->isA (kMonWarbot))
                {
                    clist.add (c);
                }
            }
        }
    }
    if (!obj->isIlkKnown ()) {
        obj->setIlkKnown ();
        I->p ("%c - %s", obj->mLetter, obj->inv ());
    }
    I->drawScreen ();
    for (int i = 0; i < clist.count (); i++)
        clist.get (i) -> newEnemy (&Hero);

    return HALFTURN;
}


static int
useProximityMine (shObject *obj)
{
    shDirection dir;
    int x = Hero.mX;
    int y = Hero.mY;

    dir = I->getDirection ();
    switch (dir) {
    case kNoDirection:
        return 0;
    case kUp:
        I->p ("You can't reach the ceiling.");
        return 0;
    case kDown:
        I->p ("Move away from this spot to place it here.");
        return 0;
    default:
        if (!Level->moveForward (dir, &x, &y)) {
            return 0;
        }
        if (Level->getCreature (x, y)) {
            I->p ("There is something standing there already.");
            return 0;
        }
        if (Level->isObstacle (x, y)) {
            I->p ("There is not enough space to place it.");
            return 0;
        }
        shMonster *mine = new shMonster (kMonSmartBomb);
        mine->mConditions |= kNoExp;
        if (!obj->isBuggy ()) {
            mine->makePet ();
            obj->setBugginessKnown ();  /* Rest in stack known. */
        }
        Hero.useUpOneObjectFromInventory (obj);
        Level->putCreature (mine, x, y);
        Level->checkTraps (x, y, 100);
    }
    return LONGTURN;
}

/* returns ms elapsed */
static int
usePortableHole (shObject *obj)
{
    shDirection dir;
    int x = Hero.mX;
    int y = Hero.mY;
    shFeature *f;
    shCreature *c;

    dir = I->getDirection ();
    switch (dir) {
    case kNoDirection:
        return 0;
    case kUp:
        I->p ("You can't reach the ceiling.");
        return 0;
    case kDown:
    default:
        if (!Level->moveForward (dir, &x, &y)) {
            return 0;
        }
        f = (shFeature *) Level->getFeature (x, y);
        if (!Level->isFloor (x, y)) {
            if (f) {
                I->p ("But there is a portable hole already.");
                return 0;
            }
            int instant = 0, nx = x, ny = y;
            shTerrainType middle = kVoid;
            /* For low depth hole activates automatically. */
            /* Patterns .#. .##. .# #. only. Pattern ### is not permitted. */
            for (int i = 0; i < 3; ++i) {
                if (!Level->moveForward (dir, &nx, &ny))
                    break;
                if (i == 0)
                    middle = Level->getSquare (nx, ny)->mTerr;
                if (Level->isFloor (nx, ny)) {
                    instant = (i < 2) || (middle == kStone);
                    break;
                }
            }
            Hero.useUpOneObjectFromInventory (obj);
            if (instant) { /* Dig immediately. */
                I->p ("As you lay it on %s it immediately activates.",
                    THE (Level->getSquare (x, y)));
                while (!Level->isFloor (x, y)) {
                    Level->dig (x, y);
                    Level->moveForward (dir, &x, &y);
                } /* Check only for hero position. */
                if (Hero.isInShop ()) {
                    Hero.damagedShop (Hero.mX, Hero.mY);
                }
                break;
            }
            shFeature *f = new shFeature ();
            f->mType = shFeature::kPortableHole;
            f->mX = x;
            f->mY = y;
            Level->addFeature (f);
            I->p ("You lay it on %s.", THE (Level->getSquare (x, y)));
            /* Laying it from outside is okay. Doing it from inside is not. */
            if (Hero.isInShop ()) {
                Hero.damagedShop (Hero.mX, Hero.mY);
            }
            break;
        }
        if (f) {
            /* It is scummable to detect traps for free otherwise. */
            I->p ("It failed to activate properly %s. It is wasted.",
                  x == Hero.mX && y == Hero.mY ? "here" : "there");
            Hero.useUpOneObjectFromInventory (obj);
            return 0;
        }
        Hero.useUpOneObjectFromInventory (obj);
        Level->addTrap (x, y, shFeature::kHole);
        if (Level->isInShop (x, y)) {
            Hero.damagedShop (x, y);
        }
        c = Level->getCreature (x, y);
        if (c && !c->isHero ()) {
            c->newEnemy (&Hero);
        }
        Level->checkTraps (x, y, 100);
    }
    return FULLTURN;
}

/* returns ms elapsed */
int
useOnOffTool (shObject *tool)
{
    if (tool->isActive ()) {
        if (tool->isBuggy ()) {
            I->p ("You can't turn off %s. It is jammed!", YOUR (tool));
            tool->setBugginessKnown ();
        } else {
            I->p ("You turn off %s.", YOUR (tool));
            tool->resetActive ();
            if (tool->isA (kObjGeigerCounter) &&
                Hero.getStoryFlag ("geiger counter message"))
            {
                I->p ("It stops clicking.");
            }
        }
    } else if (Hero.countEnergy () <= 0 && tool->myIlk ()->mEnergyUse) {
        I->p ("You're out of juice!");
    } else {
        I->p ("You turn on %s.", YOUR (tool));
        tool->setActive ();
        if (tool->isA (kObjGeigerCounter) &&
            Hero.getStoryFlag ("geiger counter message"))
        {
            if (!tool->isIlkKnown ()) {
                I->p ("It starts clicking.  It must be a Geiger counter.");
                tool->setIlkKnown ();
            } else {
                I->p ("It starts clicking.");
            }
            tool->setAppearanceKnown ();
        }
    }
    Hero.computeIntrinsics ();
    return HALFTURN;
}


int
useEnergyTank (shObject *tank)
{
    int capacity = tank->myIlk ()->mMaxCharges;
    shObject *cells = NULL;
    if (tank->mCharges == capacity) {
        I->p ("%s is full already.", YOUR (tank));
        return 0;
    }
    for (int i = 0; i < Hero.mInventory->count (); ++i) {
        cells = Hero.mInventory->get (i);
        if (cells->mIlkId == kObjEnergyCell) break;
    }
    if (!cells || cells->mIlkId != kObjEnergyCell) {
        I->p ("You have no energy cells to discharge into %s.", tank->theQuick ());
        return 0;
    }
    int eat = mini (cells->mCount, capacity - tank->mCharges);
    Hero.useUpSomeObjectsFromInventory (cells, eat);
    I->p ("%s absorbs energy of %d energy cell%s.",
        YOUR (tank), eat, eat > 1 ? "s" : "");
    tank->mCharges += eat;
    return FULLTURN;
}


static int
useKhaydarinCrystal (shObject *crystal)
{
    int worked = 0;
    if (!crystal->mCharges) {
        I->p ("Nothing happens.");
        int elapsed = crystal->isChargeKnown () ? 0 : 200;
        crystal->setChargeKnown ();
        return elapsed;
    }
    if (Hero.mPsionicStormFatigue) {
        --Hero.mPsionicStormFatigue;
    }
    if (Hero.mAbil.mPsi < Hero.mMaxAbil.mPsi) {
        I->p ("You focus your mental powers.");
        Hero.mPsiDrain = 0;
        Hero.mAbil.mPsi = Hero.mMaxAbil.mPsi;
        ++worked;
    }
    if (Hero.is (kConfused) || Hero.is (kStunned)) {
        I->p ("Your thoughts become crystal clear.");
        Hero.cure (kConfused);
        Hero.cure (kStunned);
        ++worked;
    }
    if (!worked) {
        I->p ("%s glows faintly.", YOUR (crystal));
    }
    --crystal->mCharges;
    Hero.computeSkills (); /* Charges give bonus to concentration. */
    return HALFTURN;
}


static int
useLicense (shObject *license)
{
    const char *interrogate[] =
    {
        "Orgasmatron?! What are you talking about?",
        "The Orgasmatron is rumored to be somewhere in the Mainframe.",
        "There is more than one Orgasmatron but only one true Orgasmatron.",
        "I heard someone left an Orgasmatron in a garbage compactor.",
        "Follow the white rabbit. It will lead you to the Orgasmatron.",
        "Entrance to the Mainframe is protected by a door with retina scanner.",
        "The key to retina scanner is held by the Bastard Operator From Hell.",
        "You can find the Operator at the bottom of Gamma Caves.",
        "You most probably will need to visit Gamma Caves.",
        "Some say that a skilled burglar can bypass retina scanner door.",
        "Need protection against radiation? Search the waste treatment plant.",
        "I am afraid you may need to confront Shodan about it."
    };
    const int numblurbs = sizeof (interrogate) / sizeof (char *);
    shDirection dir = I->getDirection ();
    switch (dir) {
        case kNoDirection: case kUp: case kDown:
            I->p ("No idea what to do with %s there.", YOUR (license));
            return 0;
        case kOrigin: {
            char *buf = GetBuf ();
            strcpy (buf, Hero.mName);
            buf[0] = toupper (buf[0]);
            if (!Hero.isBlind ()) {
                I->p ("It reads: \"Detective %s\".", buf);
            } else {
                I->p ("You cannot read while blind.");
            }
            return 0;
        }
        default: {
            int x = Hero.mX, y = Hero.mY;
            if (!Level->moveForward (dir, &x, &y)) {
                return 0;
            }
            shCreature *c = Level->getCreature (x, y);
            shFeature *f = Level->getFeature (x, y);
            if (c) {
                if (c->isBlind ()) {
                    I->p ("%s seems to be blind.", THE (c));
                } else if (c->is (kAsleep)) {
                    I->p ("%s snores loudly.", THE (c));
                } else if (c->isA (kMonGuardbot) && !c->isHostile ()) {
                    I->p ("You present %s to %s.", YOUR (license), THE (c));
                    shMonster *guard = (shMonster *) c;
                    if (guard->mGuard.mToll == 0) {
                        I->p ("%s ignores you.", THE (guard));
                    } else {
                        if (license->isBuggy ()) {
                            if (Hero.tryToTranslate (guard)) {
                                I->p ("\"Your license has expired. You must pay like everyone else.\"");
                            } else {
                                I->p ("%s whirs disappointingly.", THE (guard));
                            }
                        } else {
                            if (Hero.tryToTranslate (guard)) {
                                I->p ("\"You may pass.\"");
                            } else {
                                I->p ("%s beeps calmly.", THE (guard));
                            }
                            guard->mGuard.mToll = 0;
                        }
                    }
                } else if (c->isA (kMonLawyer) && c->isHostile ()) {
                    int bribe = 200 + RNG (101);
                    if (I->yn ("%s nods with understanding. "
                               "Pay $%d to buy cooperation?",
                               THE (c), bribe))
                    {
                        if (Hero.countMoney () >= bribe) {
                            I->p ("%s takes your money.", THE (c));
                            Hero.loseMoney (bribe);
                            c->gainMoney (bribe);
                            ((shMonster *) c) ->mDisposition =
                                shMonster::kIndifferent;
                        } else {
                            I->p ("You don't have that much money. %s disregards your license.", THE (c));
                        }
                    }
                } else if (c->mType != kHumanoid && c->mType != kMutant &&
                           c->mType != kOutsider && c->mType != kCyborg)
                {
                    I->p ("%s seems strangely uncooperative.", THE (c));
                } else if (!c->isHostile () || c->is (kParalyzed)) {
                    I->pageLog ();
                    I->p ("You present %s to %s.", YOUR (license), THE (c));
                    I->p ("You interrogate about the Bizarro Orgasmatron.");
                    /* This could be replaced by get random fortune message. */
                    int reply = c->mMaxHP % numblurbs;
                    if (Hero.tryToTranslate (c)) {
                        I->p ("\"%s\"", interrogate[reply]);
                    } else {
                        I->p ("You fail to understand the reply.");
                    }
                } else {
                    I->p ("%s seems more interested in killing you.", THE (c));
                }
            } else if (f && f->isDoor ()) {
                return Hero.useKey (license, f);
            } else {
                I->p ("No idea what to do with %s there.", YOUR (license));
                return 0;
            }
        }
    }
    return FULLTURN;
}

static int
swallowBluePill (shObject *pill)
{
    I->p ("You swallow the pill.");
    if (Level->isMainframe ()) return 0;
    Hero.useUpOneObjectFromInventory (pill);
    for (int i = 1; i < Maze.count (); ++i) {
        shMapLevel *L = Maze.get (i);
        if (L->isMainframe ()) {
            Level->warpCreature (&Hero, L);
            break;
        }
    }
    return 0;
}

static int
swallowRedPill (shObject *pill)
{
    I->p ("You swallow the pill.");
    if (!Level->isMainframe ()) return 0;
    Hero.useUpOneObjectFromInventory (pill);
    for (int i = 1; i < Maze.count (); ++i) {
        shMapLevel *L = Maze.get (i);
        if (L->mType == shMapLevel::kRabbit) {
            Level->warpCreature (&Hero, L);
            break;
        }
    }
    return 0;
}

static int
useTransferCable (shObject *cable)
{
    shObjectVector v;
    selectObjects (&v, Hero.mInventory, kObjGenericEnergyTank);
    selectObjects (&v, Hero.mInventory, kObjEnergyBelt);
    if (v.count () < 2) {
        I->p ("You need at least two energy cell storage items to use this.");
    }
    shObject *src = Hero.quickPickItem (&v, "transfer from", 0);
    if (!src) return 0;
    v.remove (src);
    shObject *dest = Hero.quickPickItem (&v, "transfer to", 0);
    if (!dest) return 0;
    int space = dest->myIlk ()->mMaxCharges - dest->mCharges;
    int transfer = 0;
    if (space > src->mCharges) {
        transfer = src->mCharges;
        src->mCharges = 0;
    } else {
        transfer = space;
        src->mCharges -= transfer;
    }
    int loss = cable->isOptimized () ? transfer / 100 : /* 1% */
               cable->isDebugged () ? transfer * 3 / 100 : /* 3% */
               transfer / 10; /* 10% */
    if (loss > src->mCharges) {
        transfer -= (loss - src->mCharges);
        src->mCharges = 0;
    } else {
        src->mCharges -= loss;
    }
    dest->mCharges += transfer;
    int action = dest->isUnpaid () - src->isUnpaid ();
    const char *verb =
        action <= -1 ? "steal" : action >= +1 ? "donate" : "transfer";
    if (action <= -1 && Hero.mProfession == Ninja) verb = "ninja'd";
    I->p ("You %s energy of %d cell%s.", verb, transfer + loss,
        loss > 1 ? "s" : "");
    I->p ("Energy of %d cell%s was lost in the process.", loss,
        loss > 1 ? "s" : "");
    return FULLTURN;
}

void
initializeTools ()
{
    /* enum: kObjGenericTool */
    /* enum: kObjFirstTool = kObjGenericTool */
    NEW_TOOL(kObjGenericTool,
        "tool", "tool", "tool",
        kBrightRed, kObjNothing, 0 BZ, kPlasteel,
        0, 500, kSmall, 0, 0,
        0, 0, NULL, ABSTRACT);

    /* enum: kObjGenericEnergyTank */
    NEW_TOOL(kObjGenericEnergyTank,
        "energy tank", "energy tank", "energy tank",
        kBrightYellow, kObjGenericTool, 0 BZ, kSteel,
        0, 1000, kSmall, 0, 0,
        0, 0, NULL, ABSTRACT);

    /* Weight is same as 100 energy cells. */
    /* enum: kObjSmallEnergyTank */
    NEW_TOOL(kObjSmallEnergyTank,
        "small energy tank", "small energy tank", "small energy tank",
        kBrightYellow, kObjGenericEnergyTank, 400 BZ, kSteel,
        kPreidentified | kBugProof | kChargeable, 1000, kSmall, 12, 12,
        0, 300, useEnergyTank, 25);

    /* Weight is same as 200 energy cells. */
    /* enum: kObjLargeEnergyTank */
    NEW_TOOL(kObjLargeEnergyTank,
        "large energy tank", "large energy tank", "large energy tank",
        kBrightYellow, kObjGenericEnergyTank, 1100 BZ, kSteel,
        kPreidentified | kBugProof | kChargeable, 2000, kMedium, 12, 20,
        0, 500, useEnergyTank, 15);

    /* enum: kObjGenericPlant */
    NEW_TOOL(kObjGenericPlant,
        "power plant", "power plant", "power plant",
        kBrightRed, kObjGenericTool, 1000 BZ, kPlasteel,
        kChargeable, 5000, kSmall, 12, 12,
        5000, 100, NULL, ABSTRACT);

    /* enum: kObjFissionPlant */
    NEW_TOOL(kObjFissionPlant,
        "fission power plant", "power plant", "power plant",
        kBrightRed, kObjGenericPlant, 1000 BZ, kPlasteel,
        kChargeable, 5000, kSmall, 12, 12,
        5000, 100, NULL, 20);

    /* enum: kObjFusionPlant */
    NEW_TOOL(kObjFusionPlant,
        "fusion power plant", "power plant", "power plant",
        kBrightRed, kObjGenericPlant, 2000 BZ, kPlasteel,
        kChargeable, 5000, kSmall, 12, 12,
        2500, 100, NULL, 10);

/* Left out. Its too good.
    NEW_TOOL(kObjZeroPointPlant,
        "zero point power plant", "power plant", "power plant",
        kMagenta, kObjGenericPlant, 4000 BZ, kPlasteel,
        kChargeable, 5000, kSmall, 10, 10,
        1250, 1000, NULL, 5);
*/

    /* enum: kObjPsionicShield */
    NEW_TOOL(kObjPsionicShield,
        "psionic shield generator", "power plant", "alien power plant",
        kYellow, kObjGenericPlant, 3000 BZ, kGold,
        kChargeable, 3000, kSmall, 5, 5,
        3000, 100, NULL, 1);
    AllIlks[kObjPsionicShield]->mCarriedIntrinsics |= kShielded;


    /* enum: kObjDuctTape */
    NEW_TOOL(kObjDuctTape,
        "roll of duct tape", "roll of duct tape", "roll of duct tape",
        kGray, kObjGenericTool, 5 BZ, kPlastic,
        kPreidentified | kMergeable, 100, kFine, 5, 2,
        0, 0, makeRepair, 70);

    /* enum: kObjMonkeyWrench */
    NEW_WEAPON_TOOL(kObjMonkeyWrench,
        "monkey wrench", "wrench", "wrench",
        kBrightBlue, kObjGenericTool, 20 BZ, kSteel,
        kPreidentified | kEnhanceable, 250, kTiny, 10, 1,
        newMeleeAttackP (shAttack::kClub, FULLTURN,
            100, kConcussive, 2, 7,   20, kStunning, 2, 2),
        0, 0, makeRepair, 30);
    AllIlks[kObjMonkeyWrench]->mMaxEnhancement = 4;

    /* enum: kObjFlashlight */
    NEW_TOOL(kObjFlashlight,
        "flashlight", "flashlight", "flashlight",
        kRed, kObjGenericTool, 10 BZ, kPlastic,
        kPreidentified, 200, kTiny, 2, 2,
        180000, 0, useOnOffTool, 50);
    AllIlks[kObjFlashlight]->mActiveIntrinsics |= kLightSource;

    int n = 0;
    /* enum: kObjGeigerCounter */
    NEW_TOOL(kObjGeigerCounter,
        "Geiger counter", "electronic gizmo", GizmoData[n].mDesc,
        kBrightBlue, kObjGenericTool, 300 BZ, kSteel,
        0, 500, kTiny, 10, 10,
        20000, 0, useOnOffTool, 75);

    ++n;
    /* enum: kObjMotionTracker */
    shObjectIlk *motiontracker =
    NEW_TOOL(kObjMotionTracker,
        "motion tracker", "electronic gizmo", GizmoData[n].mDesc,
        kBrightBlue, kObjGenericTool, 400 BZ, kSteel,
        0, 500, kTiny, 10, 10,
        10000, 0, useOnOffTool, 50);
    motiontracker->mActiveIntrinsics |= kMotionDetection;

    ++n;
    /* enum: kObjDroidCaller */
    NEW_TOOL(kObjDroidCaller,
        "droid caller", "electronic gizmo", GizmoData[n].mDesc,
        kBrightBlue, kObjGenericTool, 400 BZ, kSteel,
        0, 500, kTiny, 10, 10,
        0, 0, useDroidCaller, 75);

    ++n;
    /* enum: kObjTricorder */
    NEW_TOOL(kObjTricorder,
        "tricorder", "electronic gizmo", GizmoData[n].mDesc,
        kBrightBlue, kObjGenericTool, 325 BZ, kPlastic,
        0, 150, kTiny, 10, 4,
        0, 0, useTricorder, 20);


/*
    NEW_TOOL(,
        "communicator", "communicator", "communicator",
        kWhite, kObjGenericTool, 100 BZ, kSilicon,
        kPreidentified, 100, kTiny, 10, 4,
        0, 0, NULL, 25);

    NEW_TOOL(,
        "business card", "business card", "business card",
        kYellow, kObjGenericTool, 1 BZ, kPaper,
        kPreidentified, 5, kFine, 1, 1,
        0, 0, NULL, 100);
*/
    /* enum: kObjGenericKeycard */
    NEW_TOOL(kObjGenericKeycard,
        "keycard", "keycard", "keycard",
        kBrown, kObjGenericTool, 1 BZ, kPlastic,
        0, 5, kFine, 5, 1,
        0, 0, NULL, ABSTRACT);

    n = 0;
    /* enum: kObjKeycard1 */
    NEW_TOOL(kObjKeycard1,
        KeycardData[n].mDesc, "keycard", KeycardData[n].mDesc,
        KeycardData[n].mColor, kObjGenericKeycard, 10 BZ, kPlastic,
        0, 5, kFine, 5, 1,
        0, 0, useKeyTool, 17);

    ++n;
    /* enum: kObjKeycard2 */
    NEW_TOOL(kObjKeycard2,
        KeycardData[n].mDesc, "keycard", KeycardData[n].mDesc,
        KeycardData[n].mColor, kObjGenericKeycard, 20 BZ, kPlastic,
        0, 5, kFine, 5, 1,
        0, 0, useKeyTool, 17);

    ++n;
    /* enum: kObjKeycard3 */
    NEW_TOOL(kObjKeycard3,
        KeycardData[n].mDesc, "keycard", KeycardData[n].mDesc,
        KeycardData[n].mColor, kObjGenericKeycard, 30 BZ, kPlastic,
        0, 5, kFine, 5, 1,
        0, 0, useKeyTool, 17);

    ++n;
    /* enum: kObjKeycard4 */
    NEW_TOOL(kObjKeycard4,
        KeycardData[n].mDesc, "keycard", KeycardData[n].mDesc,
        KeycardData[n].mColor, kObjGenericKeycard, 50 BZ, kPlastic,
        0, 5, kFine, 5, 1,
        0, 0, useKeyTool, 17);

    ++n;
    /* enum: kObjMasterKeycard */
    NEW_TOOL(kObjMasterKeycard,
        "master keycard", "keycard", KeycardData[n].mDesc,
        KeycardData[n].mColor, kObjGenericKeycard, 150 BZ, kPlastic,
        0, 5, kFine, 5, 1,
        0, 0, useKeyTool, 17);

    /* enum: kObjLockPick */
    NEW_TOOL(kObjLockPick,
        "lock pick", "lock pick", "lock pick",
        kBrightBlue, kObjGenericTool, 200 BZ, kPlastic,
        kPreidentified, 60, kFine, 5, 5,
        0, 0, useKeyTool, 20);

    /* enum: kObjLicense */
    NEW_TOOL(kObjLicense,
        "detective license", "keycard", "detective license",
        kGray, kObjGenericTool, 10 BZ, kPlastic,
        0, 5, kFine, 5, 1,
        0, 0, useLicense, 0);

    /* enum: kObjRestrainingBolt */
    NEW_TOOL(kObjRestrainingBolt,
        "restraining bolt", "magnetic bolt", "magnetic bolt",
        kBrightBlue, kObjGenericTool, 200 BZ, kSteel,
        kPreidentified, 50, kFine, 5, 1,
        0, 0, useRestrainingBolt, 60);
    AllIlks[kObjRestrainingBolt]->mMissileAttack =
        newMissileAttack (shAttack::kAttach, FULLTURN, 10, kNoEnergy, 0, 0);
/*
    NEW_TOOL(
        "scalpel", "scalpel", "scalpel",
        kCyan, kObjGenericTool, 10, kSteel,
        kPreidentified, 50, kTiny, 10, 10,
        0, 0, NULL, 15);
*/

    /* enum: kObjMedicomp */
    NEW_TOOL(kObjMedicomp,
        "medicomp", "medicomp", "medicomp",
        kRed, kObjGenericTool, 2000 BZ, kTitanium,
        kPreidentified | kChargeable, 600, kSmall, 10, 5,
        0, 40, useMedicomp, 10);

    /* enum: kObjGenericComputer */
    NEW_TOOL(kObjGenericComputer,
        "computer", "computer", "computer",
        kGray, kObjGenericTool, 0, kSilicon,
        0, 0, kSmall, 10, 10,
        0, 0, NULL, ABSTRACT);

    /* In PRIME mini computer has many drawbacks so it is made very light
       to have some distinct advantage. */
    /* enum: kObjMiniComputer */
    NEW_TOOL(kObjMiniComputer,
        "mini computer", "mini computer", "mini computer",
        kWhite, kObjGenericComputer, 500 BZ, kPlastic,
        kPreidentified | kEnhanceable, 350, kSmall, 10, 4,
        0, 0, useComputer, 40);

    /* enum: kObjMegaComputer */
    NEW_TOOL(kObjMegaComputer,
        "mega computer", "mega computer", "mega computer",
        kWhite, kObjGenericComputer, 1000 BZ, kTitanium,
        kPreidentified | kEnhanceable, 1800, kSmall, 10, 6,
        0, 0, useComputer, 20);

    /* enum: kObjBioComputer */
    NEW_TOOL(kObjBioComputer,
        "bio computer", "lump of flesh", "bio computer",
        kRed, kObjGenericComputer, 2000 BZ, kFleshy,
        kPreidentified | kEnhanceable, 800, kSmall, 5, 10,
        0, 0, useComputer, 10);

    /* enum: kObjSatCom */
    NEW_TOOL(kObjSatCom,
        "sat-com computer", "armworn computer", "armworn computer",
        kCyan, kObjGenericComputer, 1500 BZ, kTitanium,
        kPreidentified | kEnhanceable, 600, kSmall, 10, 5,
        0, 0, useComputer, 10);

    /* enum: kObjPortableHole */
    NEW_TOOL(kObjPortableHole,
        "portable hole", "portable hole", "portable hole",
        kBlack, kObjGenericTool, 200 BZ, kCloth,
        kPreidentified | kMergeable, 0, kSmall, 100, 4,
        0, 0, usePortableHole, 50);

    /* enum: kObjProximityMine */
    NEW_TOOL(kObjProximityMine,
        "proximity mine", "mine", "proximity mine",
        kGray, kObjGenericTool, 100 BZ, kSteel,
        kPreidentified | kMergeable, 400, kFine, 10, 5,
        0, 0, useProximityMine, 50);

    /* enum: kObjKhaydarin */
    NEW_TOOL(kObjKhaydarin,
        "Khaydarin crystal shard", "crystal shard", "crystal shard",
        kBrightBlue, kObjGenericTool, 1000 BZ, kCrystal,
        kPreidentified | kBugProof | kChargeable, 40, kFine, 20, 1,
        0, 0, useKhaydarinCrystal, 2);
    AllIlks[kObjKhaydarin]->mWieldedIntrinsics |= kTelepathy;
    AllIlks[kObjKhaydarin]->mPsiModifier = +1;

    /* enum: kObjRabbitFoot */
    NEW_TOOL(kObjRabbitFoot,
        "rabbit's foot", "rabbit's foot", "rabbit's foot",
        kBrightBlue, kObjNothing, 400 BZ, kFleshy,
        kPreidentified, 50, kTiny, 10, 10,
        0, 0, NULL, 0);
    AllIlks[kObjRabbitFoot]->mCarriedIntrinsics |= kLucky;

    /* enum: kObjBluePill */
    NEW_TOOL(kObjBluePill,
        "blue pill", "pill", "blue pill",
        kBrightBlue, kObjNothing, 1 BZ, kFleshy,
        kPreidentified | kBugProof | kMergeable, 0, kFine, 0, 1,
        0, 0, swallowBluePill, 0);
    AllIlks[kObjBluePill]->mGlyph.mSym = '%';

    /* enum: kObjRedPill */
    NEW_TOOL(kObjRedPill,
        "red pill", "pill", "red pill",
        kBrightRed, kObjNothing, 1 BZ, kFleshy,
        kPreidentified | kBugProof | kMergeable, 0, kFine, 0, 1,
        0, 0, swallowRedPill, 0);
    AllIlks[kObjRedPill]->mGlyph.mSym = '%';

    /* enum: kObjTransferCable */
    NEW_TOOL(kObjTransferCable,
        "transfer cable", "cable", "transfer cable",
        kGray, kObjGenericTool, 100 BZ, kPlastic,
        kPreidentified, 50, kFine, 10, 1,
        0, 0, useTransferCable, 10);

    /* enum: kObjJunk */
    /* enum: kObjLastTool = kObjJunk */
    NEW_TOOL(kObjJunk,
        "heap of space junk", "heap of space junk", "heap of space junk",
        kBrown, kObjNothing, 5 BZ, kPlasteel,
        kPreidentified, 50000, kLarge, 10, 50,
        0, 0, NULL, 185);
}
